@@ -8,12 +8,10 @@
 }
 
 .onLoad <- function(libname, pkgname) {
-    check_cmake(get_cmake())
+    check_cmake(get_cmake(), onLoad=TRUE)
     if(is_installed() & check_os(onLoad=TRUE)) {
         # set up rkeops global options
         set_rkeops_options()
-        # use cpu by default
-        use_cpu()
     }
 }
 
