@@ -13,10 +13,10 @@ namespace keops {
 ////        Transpose of Sum : SumT< F >                   ////
 //////////////////////////////////////////////////////////////
 template<class F>
-struct Sum;
+struct Sum_Impl;
 
 template<class F, int D>
-struct SumT : UnaryOp<SumT, F, D> {
+struct SumT_Impl : UnaryOp<SumT_Impl, F, D> {
 
   static_assert(F::DIM == 1, "Dimension of input must be 1 for SumT");
 
@@ -30,10 +30,24 @@ struct SumT : UnaryOp<SumT, F, D> {
   }
 
   template<class V, class GRADIN>
-  using DiffT = typename F::template DiffT<V, Sum<GRADIN>>;
+  using DiffT = typename F::template DiffT<V, Sum_Impl<GRADIN>>;
 
 };
 
+template < class F, int D >
+struct SumT_Alias {
+	using type = SumT_Impl<F,D>;
+};
+
+template < class F >
+struct SumT_Alias<F,1> {
+	static_assert(F::DIM == 1, "Dimension of input must be 1 for SumT");
+	using type = F;
+};
+
+template < class F, int D >
+using SumT = typename SumT_Alias<F,D>::type;
+
 #define SumT(p,d) KeopsNS<SumT<decltype(InvKeopsNS(p)),d>>()
 
 }
