@@ -16,12 +16,17 @@ template < typename TYPE > DEVICE INLINE TYPE keops_cos(TYPE x) { return cos(x);
 template < typename TYPE > DEVICE INLINE TYPE keops_sin(TYPE x) { return sin(x); }
 template < typename TYPE > DEVICE INLINE TYPE keops_relu(TYPE x) { return (x<0.0f)? 0.0f : x; }
 template < typename TYPE > DEVICE INLINE TYPE keops_step(TYPE x) { return (x<0.0f)? 0.0f : 1.0f; }
-template < typename TYPE > DEVICE INLINE TYPE keops_sign(TYPE x) { return (x>0.0F)? 1.0f : ( (x<0.0f)? -1.0f : 0.0f ); }
+template < typename TYPE > DEVICE INLINE TYPE keops_sign(TYPE x) { return (x>0.0f)? 1.0f : ( (x<0.0f)? -1.0f : 0.0f ); }
+template < typename TYPE > DEVICE INLINE TYPE keops_clamp(TYPE x, TYPE a, TYPE b) { return (x<a)? a : ( (x>b)? b : x ); }
+template < typename TYPE > DEVICE INLINE TYPE keops_clampint(TYPE x, int a, int b) { return (x<a)? a : ( (x>b)? b : x ); }
+template < typename TYPE > DEVICE INLINE TYPE keops_diffclampint(TYPE x, int a, int b) { return (x<a)? 0.0f : ( (x>b)? 0.0f : 1.0f ); }
 template < typename TYPE > DEVICE INLINE TYPE keops_sqrt(TYPE x) { return sqrt(x); }
 template < typename TYPE > DEVICE INLINE TYPE keops_rsqrt(TYPE x) { return 1.0f / sqrt(x); }
+template < typename TYPE > DEVICE INLINE TYPE keops_acos(TYPE x) { return acos(x); }
+template < typename TYPE > DEVICE INLINE TYPE keops_asin(TYPE x) { return asin(x); }
+template < typename TYPE > DEVICE INLINE TYPE keops_atan(TYPE x) { return atan(x); }
 
-#ifdef __CUDA_ARCH__
-  
+#ifdef __CUDA_ARCH__  
 DEVICE INLINE float keops_pow(float x, int n) { return powf(x,n); } 
 DEVICE INLINE float keops_fma(float x, float y, float z) { return fmaf(x,y,z); } 
 DEVICE INLINE float keops_log(float x) { return logf(x); } 
@@ -32,16 +37,28 @@ DEVICE INLINE float keops_cos(float x) { return cosf(x); }
 DEVICE INLINE float keops_sin(float x) { return sinf(x); } 
 DEVICE INLINE float keops_sqrt(float x) { return sqrtf(x); } 
 DEVICE INLINE float keops_rsqrt(float x) { return rsqrtf(x); } 
-
+DEVICE INLINE float keops_acos(float x) { return acosf(x); }
+DEVICE INLINE float keops_asin(float x) { return asinf(x); }
+DEVICE INLINE float keops_atan(float x) { return atanf(x); }
 DEVICE INLINE double keops_rsqrt(double x) { return rsqrt(x); } 
    
-  #if USE_HALF 
-
+#if USE_HALF 
 DEVICE INLINE half2 keops_fma(half2 x, half2 y, half2 z) { return __hfma2(x,y,z); }
 DEVICE INLINE half2 keops_rcp(half2 x) { return h2rcp(x); } 
 DEVICE INLINE half2 keops_xlogx(half2 x) { return x * h2log(x + __heq2(x,__float2half2_rn(0.0f))); } 
 DEVICE INLINE half2 keops_relu(half2 x) { return __hlt2(__float2half2_rn(0.0f),x) * x; }  // (0<x) * x (element-wise) 
 DEVICE INLINE half2 keops_sign(half2 x) { return __hgt2(x,__float2half2_rn(0.0f)) - __hlt2(x,__float2half2_rn(0.0f)); } // (x>0) - (x<0) (element-wise)
+DEVICE INLINE half2 keops_clamp(half2 x,half2 a,half2 b) { return __hlt2(a,x)*(x-a) + a - __hlt2(b,x); } // (a<x)(x-a) + a - (b<x)(x-b) (element-wise)
+DEVICE INLINE half2 keops_clampint(half2 x,int a,int b) { 
+	half2 ah2 = __float2half2_rn((float)a);
+	half2 bh2 = __float2half2_rn((float)b);
+	return __hlt2(ah2,x)*(x-ah2) + ah2 - __hlt2(bh2,x); 
+}
+DEVICE INLINE half2 keops_diffclampint(half2 x,int a,int b) { 
+	half2 ah2 = __float2half2_rn((float)a);
+	half2 bh2 = __float2half2_rn((float)b);
+	return __hlt2(ah2,x)*__hlt2(x,bh2); 
+}
 DEVICE INLINE half2 keops_step(half2 x) { return __hgt2(x,__float2half2_rn(0.0f)); }
 DEVICE INLINE half2 keops_sqrt(half2 x) { return h2sqrt(x); }
 
